﻿# Python imports
import os
import shlex
from datetime import datetime 

# Tornado imports
import tornado.auth
import tornado.httpserver
import tornado.ioloop
import tornado.options
import tornado.web

# SqlAlchemy imports
from sqlalchemy import *
from sqlalchemy.orm import scoped_session, sessionmaker, relation, relationship
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.dialects.mssql import *

# Project imports
import handlers as h
import models
import utils

def overrides(interface_class):
    def overrider(method):
        assert(method.__name__ in dir(interface_class))
        return method
    return overrider

class ModelBinders():
    def __init__(self):
        self.model_map =  {            "categories": [CategoryBinder()],
            "customers": [CustomerBinder()],
            "customercustomerdemos": [CustomerCustomerDemoBinder()],
            "customerdemographics": [CustomerDemographicBinder()],
            "employees": [EmployeeBinder()],
            "employeeterritories": [EmployeeTerritoryBinder()],
            "orders": [OrderBinder()],
            "orderdetails": [OrderDetailBinder()],
            "products": [ProductBinder()],
            "regions": [RegionBinder()],
            "shippers": [ShipperBinder()],
            "suppliers": [SupplierBinder()],
            "territories": [TerritoryBinder()]          }

    def get_binder(self, input):
        return self.model_map[input][0]

class BaseBinder():
    def get_model(self, handler):
        return None
    def get_query(self, handler):
        return handler.db.query(self.get_model(handler))
    def create(self, handler):
        return None
    def update(self, handler):
        return None
    def delete(self, handler):
        item, related = self.fetch_using_pk(handler)
        try:
            handler.db.delete(item)
            handler.db.commit()
        except Exception, e:
            handler.db.rollback()
        return None
    def fetch_using_pk(self, handler):
        return None, dict()
    def fetch_using_filter(self, handler):
        return None
    def fetch_related(self, handler):
        return dict()
    def get_initial_pager_info(self, handler):
        # get arguments from query_string
        pager = dict()
        pager['limit'] = int(handler.get_argument('limit', 10))
        pager['offset'] = int(handler.get_argument('offset', 0))
        pager['sort'] = handler.get_argument('sort', None)
        pager['dir'] = handler.get_argument('dir', 'asc')
        return pager
    def get_base_query(self, handler, default_sort):
        # get paging information
        pager = self.get_initial_pager_info(handler)                  
        # sort records if needed
        if(pager['sort'] == None):
          pager['sort'] = default_sort
        query = self.get_query(handler).order_by(pager['sort'] + ' ' + pager['dir'])
        
        # apply filter
        filters=handler.get_argument('filter', None);
        if(filters!=None):
          i=1
          for filter in filters.split(','):
            op=handler.get_argument('fo_'+filter,None)
            c1=handler.get_argument('f1_'+filter,None)
            c2=handler.get_argument('f2_'+filter,None)
            if(op!=None):
              if(op=='null'):
                query=query.filter(filter+" IS NULL")
              if(op=='notnull'):
                query=query.filter(filter+" IS NOT NULL")
              if(op=='eq'):
                query=query.filter(filter+"="+(c1 if c1.isnumeric() else "'"+c1+"'"))
              if(op=='neq'):
                query=query.filter(filter+"!="+(c1 if c1.isnumeric() else "'"+c1+"'"))
              if(op=='like'):
                query=query.filter(filter+" like '"+c1+"'")
              if(op=='notlike'):
                query=query.filter(filter+" not like '"+c1+"'")
              if(op=='gt'):
                query=query.filter(filter+" > "+(c1 if c1.isnumeric() else "'"+c1+"'"))
              if(op=='gte'):
                query=query.filter(filter+" >= "+(c1 if c1.isnumeric() else "'"+c1+"'"))
              if(op=='lt'):
                query=query.filter(filter+" < "+(c1 if c1.isnumeric() else "'"+c1+"'"))
              if(op=='lte'):
                query=query.filter(filter+" <= "+(c1 if c1.isnumeric() else "'"+c1+"'"))
              if(op=='bet'):
                query=query.filter(filter+" >= "+(c1 if c1.isnumeric() else "'"+c1+"'"))
                query=query.filter(filter+" <= "+(c1 if c1.isnumeric() else "'"+c1+"'"))
              if(op=='cont'):
                query=query.filter(filter+" like '%"+c1+"%'")
              if(op=='start'):
                query=query.filter(filter+" like '"+c1+"%'")
              if(op=='ends'):
                query=query.filter(filter+" like '%"+c1+"'")
        return query, pager
    def run_query(self, handler, pager, query):
        # total record count
        pager['count'] = query.count()
    
        # limit and offset
        query = query.limit(pager['limit']).offset(pager['offset'])
        
        # finalize
        pager['sql'] = str(query)
        pager['dir'] = 'asc' if (pager['dir'] == 'desc') else 'desc'
        return query.all(), pager

class CategoryBinder(BaseBinder):

    @overrides(BaseBinder)
    def get_model(self, handler):
        return models.Category
    @overrides(BaseBinder)
    def fetch_using_pk(self, handler):
        return self.get_query(handler).filter('CategoryID = ' + handler.get_argument('CategoryID') + '').one(), dict()
    @overrides(BaseBinder)
    def fetch_using_filter(self, handler):
        query, pager = self.get_base_query(handler, 'CategoryID')
        # apply filter

        # return results
        return self.run_query(handler, pager, query)
    @overrides(BaseBinder)
    def fetch_related(self, handler):
        return dict()
    @overrides(BaseBinder)
    def create(self, handler):
        item = models.Category()
        try:
            item.CategoryName = handler.get_argument('categoryname')
            item.Description = handler.get_argument('description', 'NULL')
            item.Picture = handler.get_argument('picture', 'NULL')

            handler.db.add(item)
            handler.db.commit()
        except Exception, e:
            handler.db.rollback()
            raise e
        return item
    @overrides(BaseBinder)
    def update(self, handler):
        item, related = self.fetch_using_pk(handler)
        try:
            item.CategoryName = handler.get_argument('categoryname')
            item.Description = handler.get_argument('description', 'NULL')
            item.Picture = handler.get_argument('picture', 'NULL')

            handler.db.commit()
        except Exception, e:
            handler.db.rollback()
            raise e
        return item

class CustomerBinder(BaseBinder):

    @overrides(BaseBinder)
    def get_model(self, handler):
        return models.Customer
    @overrides(BaseBinder)
    def fetch_using_pk(self, handler):
        return self.get_query(handler).filter('CustomerID = \'' + handler.get_argument('CustomerID') + '\'').one(), dict()
    @overrides(BaseBinder)
    def fetch_using_filter(self, handler):
        query, pager = self.get_base_query(handler, 'CustomerID')
        # apply filter

        # return results
        return self.run_query(handler, pager, query)
    @overrides(BaseBinder)
    def fetch_related(self, handler):
        return dict()
    @overrides(BaseBinder)
    def create(self, handler):
        item = models.Customer()
        try:
            item.CustomerID = handler.get_argument('customerid')
            item.CompanyName = handler.get_argument('companyname')
            item.ContactName = handler.get_argument('contactname', 'NULL')
            item.ContactTitle = handler.get_argument('contacttitle', 'NULL')
            item.Address = handler.get_argument('address', 'NULL')
            item.City = handler.get_argument('city', 'NULL')
            item.Country = handler.get_argument('country', 'NULL')
            item.Fax = handler.get_argument('fax', 'NULL')
            item.Phone = handler.get_argument('phone', 'NULL')
            item.PostalCode = handler.get_argument('postalcode', 'NULL')
            item.Region = handler.get_argument('region', 'NULL')

            handler.db.add(item)
            handler.db.commit()
        except Exception, e:
            handler.db.rollback()
            raise e
        return item
    @overrides(BaseBinder)
    def update(self, handler):
        item, related = self.fetch_using_pk(handler)
        try:
            item.CustomerID = handler.get_argument('customerid')
            item.CompanyName = handler.get_argument('companyname')
            item.ContactName = handler.get_argument('contactname', 'NULL')
            item.ContactTitle = handler.get_argument('contacttitle', 'NULL')
            item.Address = handler.get_argument('address', 'NULL')
            item.City = handler.get_argument('city', 'NULL')
            item.Country = handler.get_argument('country', 'NULL')
            item.Fax = handler.get_argument('fax', 'NULL')
            item.Phone = handler.get_argument('phone', 'NULL')
            item.PostalCode = handler.get_argument('postalcode', 'NULL')
            item.Region = handler.get_argument('region', 'NULL')

            handler.db.commit()
        except Exception, e:
            handler.db.rollback()
            raise e
        return item

class CustomerCustomerDemoBinder(BaseBinder):

    @overrides(BaseBinder)
    def get_model(self, handler):
        return models.CustomerCustomerDemo
    @overrides(BaseBinder)
    def fetch_using_pk(self, handler):
        return self.get_query(handler).filter('CustomerID = \'' + handler.get_argument('CustomerID') + '\'' + ' and ' + 'CustomerTypeID = \'' + handler.get_argument('CustomerTypeID') + '\'').one(), dict()
    @overrides(BaseBinder)
    def fetch_using_filter(self, handler):
        query, pager = self.get_base_query(handler, 'CustomerID')
        # apply filter

        # return results
        return self.run_query(handler, pager, query)
    @overrides(BaseBinder)
    def fetch_related(self, handler):
        return dict()
    @overrides(BaseBinder)
    def create(self, handler):
        item = models.CustomerCustomerDemo()
        try:
            item.CustomerID = handler.get_argument('customerid')
            item.CustomerTypeID = handler.get_argument('customertypeid')

            handler.db.add(item)
            handler.db.commit()
        except Exception, e:
            handler.db.rollback()
            raise e
        return item
    @overrides(BaseBinder)
    def update(self, handler):
        item, related = self.fetch_using_pk(handler)
        try:
            item.CustomerID = handler.get_argument('customerid')
            item.CustomerTypeID = handler.get_argument('customertypeid')

            handler.db.commit()
        except Exception, e:
            handler.db.rollback()
            raise e
        return item

class CustomerDemographicBinder(BaseBinder):

    @overrides(BaseBinder)
    def get_model(self, handler):
        return models.CustomerDemographic
    @overrides(BaseBinder)
    def fetch_using_pk(self, handler):
        return self.get_query(handler).filter('CustomerTypeID = \'' + handler.get_argument('CustomerTypeID') + '\'').one(), dict()
    @overrides(BaseBinder)
    def fetch_using_filter(self, handler):
        query, pager = self.get_base_query(handler, 'CustomerTypeID')
        # apply filter

        # return results
        return self.run_query(handler, pager, query)
    @overrides(BaseBinder)
    def fetch_related(self, handler):
        return dict()
    @overrides(BaseBinder)
    def create(self, handler):
        item = models.CustomerDemographic()
        try:
            item.CustomerTypeID = handler.get_argument('customertypeid')
            item.CustomerDesc = handler.get_argument('customerdesc', 'NULL')

            handler.db.add(item)
            handler.db.commit()
        except Exception, e:
            handler.db.rollback()
            raise e
        return item
    @overrides(BaseBinder)
    def update(self, handler):
        item, related = self.fetch_using_pk(handler)
        try:
            item.CustomerTypeID = handler.get_argument('customertypeid')
            item.CustomerDesc = handler.get_argument('customerdesc', 'NULL')

            handler.db.commit()
        except Exception, e:
            handler.db.rollback()
            raise e
        return item

class EmployeeBinder(BaseBinder):

    @overrides(BaseBinder)
    def get_model(self, handler):
        return models.Employee
    @overrides(BaseBinder)
    def fetch_using_pk(self, handler):
        return self.get_query(handler).filter('EmployeeID = ' + handler.get_argument('EmployeeID') + '').one(), dict()
    @overrides(BaseBinder)
    def fetch_using_filter(self, handler):
        query, pager = self.get_base_query(handler, 'EmployeeID')
        # apply filter

        # return results
        return self.run_query(handler, pager, query)
    @overrides(BaseBinder)
    def fetch_related(self, handler):
        return dict()
    @overrides(BaseBinder)
    def create(self, handler):
        item = models.Employee()
        try:
            item.FirstName = handler.get_argument('firstname')
            item.LastName = handler.get_argument('lastname')
            item.TitleOfCourtesy = handler.get_argument('titleofcourtesy', 'NULL')
            item.Title = handler.get_argument('title', 'NULL')
            item.ReportsTo = handler.get_argument('reportsto', 'NULL')
            item.Extension = handler.get_argument('extension', 'NULL')
            item.HomePhone = handler.get_argument('homephone', 'NULL')
            item.Address = handler.get_argument('address', 'NULL')
            item.City = handler.get_argument('city', 'NULL')
            item.Country = handler.get_argument('country', 'NULL')
            item.Photo = handler.get_argument('photo', 'NULL')
            item.PhotoPath = handler.get_argument('photopath', 'NULL')
            item.PostalCode = handler.get_argument('postalcode', 'NULL')
            item.Region = handler.get_argument('region', 'NULL')
            item.BirthDate = 'NULL' if (handler.get_argument('birthdate', None) == None) else datetime.strptime(handler.get_argument('birthdate'), '%m/%d/%Y')
            item.HireDate = 'NULL' if (handler.get_argument('hiredate', None) == None) else datetime.strptime(handler.get_argument('hiredate'), '%m/%d/%Y')
            item.Notes = handler.get_argument('notes', 'NULL')

            handler.db.add(item)
            handler.db.commit()
        except Exception, e:
            handler.db.rollback()
            raise e
        return item
    @overrides(BaseBinder)
    def update(self, handler):
        item, related = self.fetch_using_pk(handler)
        try:
            item.FirstName = handler.get_argument('firstname')
            item.LastName = handler.get_argument('lastname')
            item.TitleOfCourtesy = handler.get_argument('titleofcourtesy', 'NULL')
            item.Title = handler.get_argument('title', 'NULL')
            item.ReportsTo = handler.get_argument('reportsto', 'NULL')
            item.Extension = handler.get_argument('extension', 'NULL')
            item.HomePhone = handler.get_argument('homephone', 'NULL')
            item.Address = handler.get_argument('address', 'NULL')
            item.City = handler.get_argument('city', 'NULL')
            item.Country = handler.get_argument('country', 'NULL')
            item.Photo = handler.get_argument('photo', 'NULL')
            item.PhotoPath = handler.get_argument('photopath', 'NULL')
            item.PostalCode = handler.get_argument('postalcode', 'NULL')
            item.Region = handler.get_argument('region', 'NULL')
            item.BirthDate = 'NULL' if (handler.get_argument('birthdate', None) == None) else datetime.strptime(handler.get_argument('birthdate'), '%m/%d/%Y')
            item.HireDate = 'NULL' if (handler.get_argument('hiredate', None) == None) else datetime.strptime(handler.get_argument('hiredate'), '%m/%d/%Y')
            item.Notes = handler.get_argument('notes', 'NULL')

            handler.db.commit()
        except Exception, e:
            handler.db.rollback()
            raise e
        return item

class EmployeeTerritoryBinder(BaseBinder):

    @overrides(BaseBinder)
    def get_model(self, handler):
        return models.EmployeeTerritory
    @overrides(BaseBinder)
    def fetch_using_pk(self, handler):
        return self.get_query(handler).filter('EmployeeID = ' + handler.get_argument('EmployeeID') + '' + ' and ' + 'TerritoryID = \'' + handler.get_argument('TerritoryID') + '\'').one(), dict()
    @overrides(BaseBinder)
    def fetch_using_filter(self, handler):
        query, pager = self.get_base_query(handler, 'EmployeeID')
        # apply filter

        # return results
        return self.run_query(handler, pager, query)
    @overrides(BaseBinder)
    def fetch_related(self, handler):
        return dict()
    @overrides(BaseBinder)
    def create(self, handler):
        item = models.EmployeeTerritory()
        try:
            item.EmployeeID = handler.get_argument('employeeid')
            item.TerritoryID = handler.get_argument('territoryid')

            handler.db.add(item)
            handler.db.commit()
        except Exception, e:
            handler.db.rollback()
            raise e
        return item
    @overrides(BaseBinder)
    def update(self, handler):
        item, related = self.fetch_using_pk(handler)
        try:
            item.EmployeeID = handler.get_argument('employeeid')
            item.TerritoryID = handler.get_argument('territoryid')

            handler.db.commit()
        except Exception, e:
            handler.db.rollback()
            raise e
        return item

class OrderBinder(BaseBinder):

    @overrides(BaseBinder)
    def get_model(self, handler):
        return models.Order
    @overrides(BaseBinder)
    def fetch_using_pk(self, handler):
        return self.get_query(handler).filter('OrderID = ' + handler.get_argument('OrderID') + '').one(), dict()
    @overrides(BaseBinder)
    def fetch_using_filter(self, handler):
        query, pager = self.get_base_query(handler, 'OrderID')
        # apply filter

        # return results
        return self.run_query(handler, pager, query)
    @overrides(BaseBinder)
    def fetch_related(self, handler):
        return dict()
    @overrides(BaseBinder)
    def create(self, handler):
        item = models.Order()
        try:
            item.CustomerID = handler.get_argument('customerid', 'NULL')
            item.EmployeeID = handler.get_argument('employeeid', 'NULL')
            item.OrderDate = 'NULL' if (handler.get_argument('orderdate', None) == None) else datetime.strptime(handler.get_argument('orderdate'), '%m/%d/%Y')
            item.RequiredDate = 'NULL' if (handler.get_argument('requireddate', None) == None) else datetime.strptime(handler.get_argument('requireddate'), '%m/%d/%Y')
            item.Freight = handler.get_argument('freight', 'NULL')
            item.ShipAddress = handler.get_argument('shipaddress', 'NULL')
            item.ShipCity = handler.get_argument('shipcity', 'NULL')
            item.ShipCountry = handler.get_argument('shipcountry', 'NULL')
            item.ShipName = handler.get_argument('shipname', 'NULL')
            item.ShippedDate = 'NULL' if (handler.get_argument('shippeddate', None) == None) else datetime.strptime(handler.get_argument('shippeddate'), '%m/%d/%Y')
            item.ShipPostalCode = handler.get_argument('shippostalcode', 'NULL')
            item.ShipRegion = handler.get_argument('shipregion', 'NULL')
            item.ShipVia = handler.get_argument('shipvia', 'NULL')

            handler.db.add(item)
            handler.db.commit()
        except Exception, e:
            handler.db.rollback()
            raise e
        return item
    @overrides(BaseBinder)
    def update(self, handler):
        item, related = self.fetch_using_pk(handler)
        try:
            item.CustomerID = handler.get_argument('customerid', 'NULL')
            item.EmployeeID = handler.get_argument('employeeid', 'NULL')
            item.OrderDate = 'NULL' if (handler.get_argument('orderdate', None) == None) else datetime.strptime(handler.get_argument('orderdate'), '%m/%d/%Y')
            item.RequiredDate = 'NULL' if (handler.get_argument('requireddate', None) == None) else datetime.strptime(handler.get_argument('requireddate'), '%m/%d/%Y')
            item.Freight = handler.get_argument('freight', 'NULL')
            item.ShipAddress = handler.get_argument('shipaddress', 'NULL')
            item.ShipCity = handler.get_argument('shipcity', 'NULL')
            item.ShipCountry = handler.get_argument('shipcountry', 'NULL')
            item.ShipName = handler.get_argument('shipname', 'NULL')
            item.ShippedDate = 'NULL' if (handler.get_argument('shippeddate', None) == None) else datetime.strptime(handler.get_argument('shippeddate'), '%m/%d/%Y')
            item.ShipPostalCode = handler.get_argument('shippostalcode', 'NULL')
            item.ShipRegion = handler.get_argument('shipregion', 'NULL')
            item.ShipVia = handler.get_argument('shipvia', 'NULL')

            handler.db.commit()
        except Exception, e:
            handler.db.rollback()
            raise e
        return item

class OrderDetailBinder(BaseBinder):

    @overrides(BaseBinder)
    def get_model(self, handler):
        return models.OrderDetail
    @overrides(BaseBinder)
    def fetch_using_pk(self, handler):
        return self.get_query(handler).filter('OrderID = ' + handler.get_argument('OrderID') + '' + ' and ' + 'ProductID = ' + handler.get_argument('ProductID') + '').one(), dict()
    @overrides(BaseBinder)
    def fetch_using_filter(self, handler):
        query, pager = self.get_base_query(handler, 'OrderID')
        # apply filter

        # return results
        return self.run_query(handler, pager, query)
    @overrides(BaseBinder)
    def fetch_related(self, handler):
        return dict()
    @overrides(BaseBinder)
    def create(self, handler):
        item = models.OrderDetail()
        try:
            item.OrderID = handler.get_argument('orderid')
            item.ProductID = handler.get_argument('productid')
            item.Quantity = handler.get_argument('quantity')
            item.UnitPrice = handler.get_argument('unitprice')
            item.Discount = handler.get_argument('discount')

            handler.db.add(item)
            handler.db.commit()
        except Exception, e:
            handler.db.rollback()
            raise e
        return item
    @overrides(BaseBinder)
    def update(self, handler):
        item, related = self.fetch_using_pk(handler)
        try:
            item.OrderID = handler.get_argument('orderid')
            item.ProductID = handler.get_argument('productid')
            item.Quantity = handler.get_argument('quantity')
            item.UnitPrice = handler.get_argument('unitprice')
            item.Discount = handler.get_argument('discount')

            handler.db.commit()
        except Exception, e:
            handler.db.rollback()
            raise e
        return item

class ProductBinder(BaseBinder):

    @overrides(BaseBinder)
    def get_model(self, handler):
        return models.Product
    @overrides(BaseBinder)
    def fetch_using_pk(self, handler):
        return self.get_query(handler).filter('ProductID = ' + handler.get_argument('ProductID') + '').one(), dict()
    @overrides(BaseBinder)
    def fetch_using_filter(self, handler):
        query, pager = self.get_base_query(handler, 'ProductID')
        # apply filter

        # return results
        return self.run_query(handler, pager, query)
    @overrides(BaseBinder)
    def fetch_related(self, handler):
        return dict()
    @overrides(BaseBinder)
    def create(self, handler):
        item = models.Product()
        try:
            item.CategoryID = handler.get_argument('categoryid', 'NULL')
            item.SupplierID = handler.get_argument('supplierid', 'NULL')
            item.ProductName = handler.get_argument('productname')
            item.UnitPrice = handler.get_argument('unitprice', 'NULL')
            item.UnitsInStock = handler.get_argument('unitsinstock', 'NULL')
            item.UnitsOnOrder = handler.get_argument('unitsonorder', 'NULL')
            item.QuantityPerUnit = handler.get_argument('quantityperunit', 'NULL')
            item.Discontinued = False if (handler.get_argument('discontinued', None) == None) else (handler.get_argument('discontinued')=='discontinued')
            item.ReorderLevel = handler.get_argument('reorderlevel', 'NULL')

            handler.db.add(item)
            handler.db.commit()
        except Exception, e:
            handler.db.rollback()
            raise e
        return item
    @overrides(BaseBinder)
    def update(self, handler):
        item, related = self.fetch_using_pk(handler)
        try:
            item.CategoryID = handler.get_argument('categoryid', 'NULL')
            item.SupplierID = handler.get_argument('supplierid', 'NULL')
            item.ProductName = handler.get_argument('productname')
            item.UnitPrice = handler.get_argument('unitprice', 'NULL')
            item.UnitsInStock = handler.get_argument('unitsinstock', 'NULL')
            item.UnitsOnOrder = handler.get_argument('unitsonorder', 'NULL')
            item.QuantityPerUnit = handler.get_argument('quantityperunit', 'NULL')
            item.Discontinued = False if (handler.get_argument('discontinued', None) == None) else (handler.get_argument('discontinued')=='discontinued')
            item.ReorderLevel = handler.get_argument('reorderlevel', 'NULL')

            handler.db.commit()
        except Exception, e:
            handler.db.rollback()
            raise e
        return item

class RegionBinder(BaseBinder):

    @overrides(BaseBinder)
    def get_model(self, handler):
        return models.Region
    @overrides(BaseBinder)
    def fetch_using_pk(self, handler):
        return self.get_query(handler).filter('RegionID = ' + handler.get_argument('RegionID') + '').one(), dict()
    @overrides(BaseBinder)
    def fetch_using_filter(self, handler):
        query, pager = self.get_base_query(handler, 'RegionID')
        # apply filter

        # return results
        return self.run_query(handler, pager, query)
    @overrides(BaseBinder)
    def fetch_related(self, handler):
        return dict()
    @overrides(BaseBinder)
    def create(self, handler):
        item = models.Region()
        try:
            item.RegionID = handler.get_argument('regionid')
            item.RegionDescription = handler.get_argument('regiondescription')

            handler.db.add(item)
            handler.db.commit()
        except Exception, e:
            handler.db.rollback()
            raise e
        return item
    @overrides(BaseBinder)
    def update(self, handler):
        item, related = self.fetch_using_pk(handler)
        try:
            item.RegionID = handler.get_argument('regionid')
            item.RegionDescription = handler.get_argument('regiondescription')

            handler.db.commit()
        except Exception, e:
            handler.db.rollback()
            raise e
        return item

class ShipperBinder(BaseBinder):

    @overrides(BaseBinder)
    def get_model(self, handler):
        return models.Shipper
    @overrides(BaseBinder)
    def fetch_using_pk(self, handler):
        return self.get_query(handler).filter('ShipperID = ' + handler.get_argument('ShipperID') + '').one(), dict()
    @overrides(BaseBinder)
    def fetch_using_filter(self, handler):
        query, pager = self.get_base_query(handler, 'ShipperID')
        # apply filter

        # return results
        return self.run_query(handler, pager, query)
    @overrides(BaseBinder)
    def fetch_related(self, handler):
        return dict()
    @overrides(BaseBinder)
    def create(self, handler):
        item = models.Shipper()
        try:
            item.CompanyName = handler.get_argument('companyname')
            item.Phone = handler.get_argument('phone', 'NULL')

            handler.db.add(item)
            handler.db.commit()
        except Exception, e:
            handler.db.rollback()
            raise e
        return item
    @overrides(BaseBinder)
    def update(self, handler):
        item, related = self.fetch_using_pk(handler)
        try:
            item.CompanyName = handler.get_argument('companyname')
            item.Phone = handler.get_argument('phone', 'NULL')

            handler.db.commit()
        except Exception, e:
            handler.db.rollback()
            raise e
        return item

class SupplierBinder(BaseBinder):

    @overrides(BaseBinder)
    def get_model(self, handler):
        return models.Supplier
    @overrides(BaseBinder)
    def fetch_using_pk(self, handler):
        return self.get_query(handler).filter('SupplierID = ' + handler.get_argument('SupplierID') + '').one(), dict()
    @overrides(BaseBinder)
    def fetch_using_filter(self, handler):
        query, pager = self.get_base_query(handler, 'SupplierID')
        # apply filter

        # return results
        return self.run_query(handler, pager, query)
    @overrides(BaseBinder)
    def fetch_related(self, handler):
        return dict()
    @overrides(BaseBinder)
    def create(self, handler):
        item = models.Supplier()
        try:
            item.CompanyName = handler.get_argument('companyname')
            item.ContactName = handler.get_argument('contactname', 'NULL')
            item.ContactTitle = handler.get_argument('contacttitle', 'NULL')
            item.HomePage = handler.get_argument('homepage', 'NULL')
            item.Address = handler.get_argument('address', 'NULL')
            item.City = handler.get_argument('city', 'NULL')
            item.Country = handler.get_argument('country', 'NULL')
            item.Fax = handler.get_argument('fax', 'NULL')
            item.Phone = handler.get_argument('phone', 'NULL')
            item.PostalCode = handler.get_argument('postalcode', 'NULL')
            item.Region = handler.get_argument('region', 'NULL')

            handler.db.add(item)
            handler.db.commit()
        except Exception, e:
            handler.db.rollback()
            raise e
        return item
    @overrides(BaseBinder)
    def update(self, handler):
        item, related = self.fetch_using_pk(handler)
        try:
            item.CompanyName = handler.get_argument('companyname')
            item.ContactName = handler.get_argument('contactname', 'NULL')
            item.ContactTitle = handler.get_argument('contacttitle', 'NULL')
            item.HomePage = handler.get_argument('homepage', 'NULL')
            item.Address = handler.get_argument('address', 'NULL')
            item.City = handler.get_argument('city', 'NULL')
            item.Country = handler.get_argument('country', 'NULL')
            item.Fax = handler.get_argument('fax', 'NULL')
            item.Phone = handler.get_argument('phone', 'NULL')
            item.PostalCode = handler.get_argument('postalcode', 'NULL')
            item.Region = handler.get_argument('region', 'NULL')

            handler.db.commit()
        except Exception, e:
            handler.db.rollback()
            raise e
        return item

class TerritoryBinder(BaseBinder):

    @overrides(BaseBinder)
    def get_model(self, handler):
        return models.Territory
    @overrides(BaseBinder)
    def fetch_using_pk(self, handler):
        return self.get_query(handler).filter('TerritoryID = \'' + handler.get_argument('TerritoryID') + '\'').one(), dict()
    @overrides(BaseBinder)
    def fetch_using_filter(self, handler):
        query, pager = self.get_base_query(handler, 'TerritoryID')
        # apply filter

        # return results
        return self.run_query(handler, pager, query)
    @overrides(BaseBinder)
    def fetch_related(self, handler):
        return dict()
    @overrides(BaseBinder)
    def create(self, handler):
        item = models.Territory()
        try:
            item.TerritoryID = handler.get_argument('territoryid')
            item.RegionID = handler.get_argument('regionid')
            item.TerritoryDescription = handler.get_argument('territorydescription')

            handler.db.add(item)
            handler.db.commit()
        except Exception, e:
            handler.db.rollback()
            raise e
        return item
    @overrides(BaseBinder)
    def update(self, handler):
        item, related = self.fetch_using_pk(handler)
        try:
            item.TerritoryID = handler.get_argument('territoryid')
            item.RegionID = handler.get_argument('regionid')
            item.TerritoryDescription = handler.get_argument('territorydescription')

            handler.db.commit()
        except Exception, e:
            handler.db.rollback()
            raise e
        return item
